@@ -7,10 +7,12 @@ from metamake._bootstrap import serialize_metamake_to_makefile
 from metamake._flags import Flag
 from metamake._dependencies import Dependency
 from metamake._shell import shell, ShellCommandFailedError
+from metamake._namespaces import namespace, get_current_namespace
 
 __all__ = [
     
     "task",
+    "namespace",
     "shell"
     "path",
     "Flag",
@@ -22,7 +24,7 @@ __all__ = [
     "run_default_task",
 ]
 
-__version__ = "1.1.0"
+__version__ = "1.1.1"
 
 _DEFAULT_STDERR_LOGS = ["metamake.filesystem", "metamake.shell"]
 _all_tasks = {}
@@ -31,6 +33,7 @@ _first_task_name = None
 _default_task_name = None
 _did_setup_stderr_logging = False
 _bootstrap_filename = None
+_defined_flags = False
 
 def bootstrap(bootstrap_filename):
     """call this function to set a bootstrap filename for
@@ -42,7 +45,7 @@ def task(method=None, default=False):
     """this decorator should be used to mark Python functions
     in your Makefile.py as tasks that can be executed on
     the commandline.  You can specify 'default = True' if
-    you want that task to be the default"""
+    you want that task to be the default."""
     def decorate(method, default):
 
         global _first_task_name
@@ -52,10 +55,18 @@ def task(method=None, default=False):
             _first_task_name = method.__name__
         
         if default == True:
+            # TODO: how should we handle overwriting of the default task?
             _default_task_name = method.__name__
         
-        _all_tasks[method.__name__] = method
-        _ordered_task_keys.append(method.__name__)
+        # index under the function name and namespace
+        namespace_list = get_current_namespace()
+        if namespace_list:
+            lookup_name = ":".join([x.__name__ for x in (namespace_list + [method])])
+        else:
+            lookup_name = method.__name__
+        _all_tasks[lookup_name] = method
+        _ordered_task_keys.append(lookup_name)
+        
         return method
         
     if method == None:
@@ -68,6 +79,7 @@ def task(method=None, default=False):
 def list_tasks():
     """lists all tasks that have been defined in your
     Makefile.py file"""
+    _define_flags()
     _ensure_stderr_logs_are_active()
     if _bootstrap_filename:
         serialize_metamake_to_makefile(_bootstrap_filename)
@@ -84,17 +96,19 @@ def list_tasks():
                 shortdoc = shortdoc.split("\n")[0] + "..."
             format_string = "%%(name)-%ss # %%(optional_default_doc)s%%(doc)s" % max([len(name)+1 for name in _all_tasks.keys()])
             print format_string % dict(
-                name = task.__name__,
-                doc = shortdoc,
-                optional_default_doc = {_default_task_name: "(default) "}.get(task_name, ""),
+                name=task_name,
+                doc=shortdoc,
+                optional_default_doc={_default_task_name: "(default) "}.get(task_name, ""),
             )
-    print
-    for flagname, explanation in Flag._flag_explanation_lookup.iteritems():
-        print "[%(flagname)s=?] %(explanation)s" % locals()
+    if len(Flag._flag_explanation_lookup) > 0:
+        print
+        for flagname, explanation in Flag._flag_explanation_lookup.iteritems():
+            print "[%(flagname)s=?] %(explanation)s" % locals()
     print
 
-def run_task(task_name):
+def run_task(task_name, from_makefile=False):
     """runs a task (by name)"""
+    _define_flags()    
     _ensure_stderr_logs_are_active()
     if _bootstrap_filename:
         serialize_metamake_to_makefile(_bootstrap_filename)
@@ -106,11 +120,25 @@ def run_task(task_name):
         try:
             _all_tasks[task_name]()
         except Exception, e:
-            print str(e)
-            if Flag("tracebacks").given and Flag("tracebacks").value is True:
+            
+            # print the error
+            print "*** error: %s, %s" % (e.__class__.__name__, str(e))
+            
+            # show a full traceback if desired
+            # if Flag("tracebacks").given and Flag("tracebacks").value is True:
+            if "--trace" in sys.argv or (Flag("mmtrace").given and Flag("mmtrace").value == True):
                 print "================================"
                 import traceback
                 traceback.print_tb(sys.exc_info()[2])
+            else:
+                # tell the user they can get a traceback
+                if from_makefile:
+                    # use mmtrace=1 as detail flag
+                    print "(use mmtrace=1 for more detail)"
+                else:
+                    # use --trace as detail flag
+                    print "(use --trace for more detail)"
+                    
             raise SystemExit(1)
     else:
         print >> sys.stderr, "*** there is no task with the name '%s'" % task_name
@@ -125,7 +153,16 @@ def run_default_task():
         run_task(_default_task_name)
     
     else:
-        run_task(_first_task_name)
+        # by default, list all tasks if no other default task has been set
+        list_tasks()
+
+def _define_flags():
+    """helper method to ensure that the global flags are set"""
+    global _defined_flags
+    if not _defined_flags:
+        pass
+        # Flag("tracebacks").explain("set to 'true' for full tracebacks on error", convert=bool, allow=[True, False])
+        # _defined_flags = True
 
 def _ensure_stderr_logs_are_active():
     """helper method to ensure that file system actions
@@ -141,8 +178,6 @@ def _ensure_stderr_logs_are_active():
 
 def metamake_bin():
     """the actual metamake commandline tool (main function)"""
-
-    Flag("tracebacks").explain("if you want to see full Python tracebacks upon error, set this flag to true", convert=bool, allow=[True, False])
     
     import sys
     
